<?PHP

/*
 +--------------------------------------------------------------------+
 | CiviCRM version 2.1                                                |
 +--------------------------------------------------------------------+
 | Copyright CiviCRM LLC (c) 2004-2008                                |
 +--------------------------------------------------------------------+
 | This file is a part of CiviCRM.                                    |
 |                                                                    |
 | CiviCRM is free software; you can copy, modify, and distribute it  |
 | under the terms of the GNU Affero General Public License           |
 | Version 3, 19 November 2007.                                       |
 |                                                                    |
 | CiviCRM is distributed in the hope that it will be useful, but     |
 | WITHOUT ANY WARRANTY; without even the implied warranty of         |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.               |
 | See the GNU Affero General Public License for more details.        |
 |                                                                    |
 | You should have received a copy of the GNU Affero General Public   |
 | License along with this program; if not, contact CiviCRM LLC       |
 | at info[AT]civicrm[DOT]org. If you have questions about the        |
 | GNU Affero General Public License or the licensing of CiviCRM,     |
 | see the CiviCRM license FAQ at http://civicrm.org/licensing        |
 +--------------------------------------------------------------------+
*/

/**
 *
 *
 * @package CRM
 * @copyright CiviCRM LLC (c) 2004-2007
 * $Id$
 *
 */


/**
 * This class generates form components for processing a participation fee block 
 */
class CRM_Event_Form_EventFees
{
    /** 
     * Function to set variables up before form is built 
     *                                                           
     * @return void 
     * @access public 
     */ 
    static function preProcess( &$form )  
    {
        $form->_eventId       = CRM_Utils_Request::retrieve( 'eventId', 'Positive', $form );
        $form->_participantId = CRM_Utils_Request::retrieve( 'participantId', 'Positive', $form );
        $form->_discountId    = CRM_Utils_Request::retrieve( 'discountId', 'Positive', $form );
    }
    
    /**
     * This function sets the default values for the form in edit/view mode
     * the default values are retrieved from the database
     * 
     * @access public
     * @return None
     */
    static function setDefaultValues( &$form ) 
    { 
        $defaults = array( );
        
        if ( $form->_participantId ) {
            $ids    = array( );
            $params = array( 'id' => $form->_participantId );
            
            require_once "CRM/Event/BAO/Participant.php";
            CRM_Event_BAO_Participant::getValues( $params, $defaults, $ids );            
            $defaults[$form->_participantId]['send_receipt'] = 0;
        } else {
            $defaults[$form->_participantId]['send_receipt'] = 1;
            if ( $form->_eventId ) {
                $defaults[$form->_participantId]['receipt_text'] = CRM_Core_DAO::getFieldValue( 'CRM_Event_DAO_EventPage',
                                                                                                $form->_eventId, 
                                                                                                'confirm_email_text',
                                                                                                'event_id');
            }
            $today_date = getDate();
            $defaults[$form->_participantId]['receive_date']['M'] = $today_date['mon'];
            $defaults[$form->_participantId]['receive_date']['d'] = $today_date['mday'];
            $defaults[$form->_participantId]['receive_date']['Y'] = $today_date['year'];
        }
        if ( $form->_mode ) {
            $fields = array( );
            
            foreach ( $form->_fields as $name => $dontCare ) {
                $fields[$name] = 1;
            }
            
            $names = array("first_name", "middle_name", "last_name" );
            foreach ($names as $name) {
                $fields[$name] = 1;
            }
            
            $fields["state_province-{$form->_bltID}"] = 1;
            $fields["country-{$form->_bltID}"       ] = 1;
            $fields["email-{$form->_bltID}"         ] = 1;
            $fields["email-Primary"                 ] = 1;
            
            require_once "CRM/Core/BAO/UFGroup.php";
            CRM_Core_BAO_UFGroup::setProfileDefaults( $form->_contactID, $fields, $form->_defaults );
            
            $defaultAddress = array("street_address-5","city-5", "state_province_id-5", "country_id-5","postal_code-5" );
            foreach ($defaultAddress as $name) {
                if ( ! empty( $form->_defaults[$name] ) ) {
                    $defaults[$form->_participantId][$name] = $form->_defaults[$name];
                }
            } 
            
            foreach ($names as $name) {
                if ( ! empty( $form->_defaults[$name] ) ) {
                    $defaults[$form->_participantId]["billing_" . $name] = $form->_defaults[$name];
                }
            } 
        }
        
        require_once 'CRM/Core/BAO/PriceSet.php';
        if ( $priceSetId = CRM_Core_BAO_PriceSet::getFor( 'civicrm_event_page', $form->_eventId ) ) {
            $fields = array( );
    
            $eventLevel = explode( CRM_Core_BAO_CustomOption::VALUE_SEPERATOR, 
                                   substr( $defaults[$form->_participantId]['fee_level'], 1, -1 ) );
            foreach ( $eventLevel as $id => $name ) {
                $optionValue         = new CRM_Core_BAO_OptionValue( );
                $optionValue->label  = $name;
                $optionValue->find( true );
                
                if ($optionValue->option_group_id ){
                    $groupName       = CRM_Core_DAO::getFieldValue( 'CRM_Core_DAO_OptionGroup', $optionValue->option_group_id, 'name' );
                    $fieldName       = CRM_Core_DAO::getFieldValue( 'CRM_Core_DAO_PriceField', substr( $groupName, -1, 1 ), 'label') ;
                    $eventLevel[$id] = array( 'fieldName'   => $fieldName,
                                              'optionLabel' => $name );
                }
            }
            //for the texfield default value
            foreach ( $eventLevel as $id => $values ) {
                if( !is_array( $values ) ){
                    $textLevel       = explode( ' - ', $values );
                    $eventLevel[$id] = array( 'fieldName'   => $textLevel[0],
                                              'optionLabel' => $textLevel[1] );
                }       
            }
            
            require_once 'CRM/Core/BAO/PriceField.php';
            foreach ( $eventLevel as $values ) {
                $priceField        = new CRM_Core_BAO_PriceField( );
                $priceField->label = $values['fieldName'];
                
                $priceField->find( true );
                
                // FIXME: we are not storing qty for text type (for
                // offline mode). Hence cannot set defaults for Text
                // type price field
                if ( $priceField->html_type == 'Text' ) {
                    $defaults[$form->_participantId]["price_{$priceField->id}"] = $values['optionLabel'];
                    continue;
                }
                
                $optionId = CRM_Core_BAO_PriceField::getOptionId( $values['optionLabel'], $priceField->id );
                
                if ( $priceField->html_type == 'CheckBox' ) {
                    $defaults[$form->_participantId]["price_{$priceField->id}"][$optionId] = 1;
                    continue;
                }
                $defaults[$form->_participantId]["price_{$priceField->id}"] = $optionId;
            }
        } else {
            $optionGroupId = null;

            // if user has selected discount use that to set default
            if ( isset( $form->_discountId ) ) {
                $defaults[$form->_participantId]['discount_id'] = $form->_discountId;

                //hack to set defaults for already selected discount value
                if ( $form->_action == CRM_Core_Action::UPDATE && !$form->_originalDiscountId ) {
                    $form->_originalDiscountId = $defaults[$form->_participantId]['discount_id'];
                    if ( $form->_originalDiscountId ) {
                        $optionGroupId = CRM_Core_DAO::getFieldValue( "CRM_Core_DAO_Discount", 
                                                                      $form->_originalDiscountId,
                                                                      'option_group_id' );
                        $defaults[$form->_participantId]['discount_id'] = $form->_originalDiscountId;
                    }
                }
            } 

            if ( ( $form->_action == CRM_Core_Action::ADD ) ) {
                // this case is for add mode, where we show discount automatically
                if ( !isset( $form->_discountId ) ) {
                    require_once 'CRM/Core/BAO/Discount.php';
                    $discountId = CRM_Core_BAO_Discount::findSet( $form->_eventId, 'civicrm_event' );
                } else {
                    $discountId = $form->_discountId;
                }

                if ( $form->_eventId && $discountId ) {
                    $defaultDiscountId = CRM_Core_DAO::getFieldValue( "CRM_Event_DAO_EventPage", 
                                                                      $form->_eventId, 
                                                                      'default_discount_id' );
                    if ( $defaultDiscountId ) {
                        $discountKey = CRM_Core_DAO::getFieldValue( "CRM_Core_DAO_OptionValue", 
                                                                    $defaultDiscountId, 
                                                                    'weight' );
                    }
                    
                    $defaults[$form->_participantId]['discount_id'] = $discountId;
                    $defaults[$form->_participantId]['amount']      = $form->_values['discount'][$discountId]['amount_id'][$discountKey];

                    $optionGroupId = CRM_Core_DAO::getFieldValue( "CRM_Core_DAO_Discount", 
                                                                  $discountId,
                                                                  'option_group_id' );
                } else {                    
                    if ( $form->_eventId ) {
                        $defaults[$form->_participantId]['amount'] = CRM_Core_DAO::getFieldValue( "CRM_Event_DAO_EventPage", 
                                                                                                  $form->_eventId, 
                                                                                                  'default_fee_id', 
                                                                                                  'event_id' );
                    }
                }
            }

            if ( ($form->_action == CRM_Core_Action::UPDATE ) && CRM_Utils_Array::value( 'event_id', $defaults[$form->_participantId] ) ) {
                if ( ! $optionGroupId ) {
                    $optionGroupId = CRM_Core_DAO::getFieldValue( "CRM_Core_DAO_OptionGroup", 
                                                                  'civicrm_event_page.amount.' . 
                                                                  CRM_Core_DAO::getFieldValue( "CRM_Event_DAO_EventPage", 
                                                                                               $defaults[$form->_participantId]['event_id'],
                                                                                               'id', 'event_id' ), 
                                                                  'id', 
                                                                  'name' );
                }

                $optionParams = array( 'option_group_id' => $optionGroupId,
                                       'label'           => CRM_Utils_Array::value('fee_level',$defaults[$form->_participantId]) );
                
                CRM_Core_BAO_CustomOption::retrieve( $optionParams, $params );
                $defaults[$form->_participantId]['amount'] = $params['id'];
            }
        }
       
        return $defaults[$form->_participantId];
        
    }
    
    /** 
     * Function to build the form 
     * 
     * @return None 
     * @access public 
     */ 
    static function buildQuickForm( &$form )  
    {
        if ( $form->_eventId ) {
            $form->_isPaidEvent = CRM_Core_DAO::getFieldValue( 'CRM_Event_DAO_Event', $form->_eventId, 'is_monetary' );
            $form->addElement( 'hidden', 'hidden_feeblock', 1 );
        }
        
        if ( $form->_participantId ) { 
            if ( CRM_Core_DAO::getFieldValue( 'CRM_Event_DAO_ParticipantPayment', 
                                              $form->_participantId, 'contribution_id', 'participant_id' ) ) {
                $form->_online = true;
            }
        }
        
        if ( $form->_isPaidEvent ) {
            require_once "CRM/Event/BAO/EventPage.php";
            $params = array( 'event_id' => $form->_eventId );
            CRM_Event_BAO_EventPage::retrieve( $params, $eventPage );

            //retrieve custom information
            $form->_values = array( );
            require_once "CRM/Event/Form/Registration/Register.php";
            CRM_Event_Form_Registration::initPriceSet($form, $eventPage['id'] );
            CRM_Event_Form_Registration_Register::buildAmount( $form, true, $form->_discountId );
            $discounts = array( );
            if ( !empty( $form->_values['discount'] ) ) {
                foreach( $form->_values['discount'] as $key => $value ) { 
                    $discounts[$key] = $value['name'];                   
                }
                
                $element = $form->add('select', 'discount_id', 
                                      ts( 'Discount Set' ), 
                                      array( 0 => ts( '- select -' )) + $discounts,
                                      false,
                                      array('onchange' => "buildFeeBlock( {$form->_eventId}, this.value );") );
           
                if ( $form->_online ) {
                    $element->freeze();
                }
            }
            if ( $form->_mode ) {
                require_once 'CRM/Core/Payment/Form.php';
                CRM_Core_Payment_Form::buildCreditCard( $form, true );
            } else if ( !$form->_mode ) {
                $form->addElement('checkbox', 'record_contribution', ts('Record Payment?'), null, 
                                  array('onclick' =>"return showHideByValue('record_contribution','','payment_information','table-row','radio',false);"));
                
                require_once 'CRM/Contribute/PseudoConstant.php';
                $form->add('select', 'contribution_type_id', 
                           ts( 'Contribution Type' ), 
                           array(''=>ts( '- select -' )) + CRM_Contribute_PseudoConstant::contributionType( ) );
                
                $form->add('date', 'receive_date', ts('Received'), CRM_Core_SelectValues::date('activityDate'), false );         
                $form->addRule('receive_date', ts('Select a valid date.'), 'qfDate');
                
                $form->add('select', 'payment_instrument_id', 
                           ts( 'Paid By' ), 
                           array(''=>ts( '- select -' )) + CRM_Contribute_PseudoConstant::paymentInstrument( )
                           );
                
                // don't show transaction id in batch update mode
                $path = CRM_Utils_System::currentPath( );
                $form->assign('showTransactionId', false );
                if ( $path != 'civicrm/contact/search/basic' ) {
                    $form->add('text', 'trxn_id', ts('Transaction ID'));
                    $form->addRule( 'trxn_id', ts('Transaction ID already exists in Database.'),
                                    'objectExists', array( 'CRM_Contribute_DAO_Contribution', $form->_id, 'trxn_id' ) );
                    $form->assign('showTransactionId', true );
                }
            
                $form->add('select', 'contribution_status_id',
                           ts('Payment Status'), 
                           CRM_Contribute_PseudoConstant::contributionStatus( )
                           );
            }
        } else {
            $form->add( 'text', 'amount', ts('Event Fee(s)') );
        }
        
        $form->assign("paid", $form->_isPaidEvent );
        
        $form->addElement('checkbox', 
                          'send_receipt', 
                          ts('Send Confirmation?'), null, 
                          array('onclick' =>"return showHideByValue('send_receipt','','notice','block','radio',false);") );
        $form->add('textarea', 'receipt_text', ts('Confirmation Message') );
        
        // Retrieve the name and email of the contact - form will be the TO for receipt email
        if ( $form->_contactID ) {
            list( $form->_contributorDisplayName, 
                 $form->_contributorEmail ) = CRM_Contact_BAO_Contact_Location::getEmailDetails( $form->_contactID );
            $form->assign( 'email', $form->_contributorEmail );
        } else {
            //show email block for batch update for event
            $form->assign( 'batchEmail', true );
        }
    }
}

